//Support_AdminEvents.cs
//Adam487 (Forum: Headcrab Zombie)
//This file is freely available for use by others. I only ask that you leave this comment header unchanged,
//and do not make changes that may cause interference with other add-ons running this support script

//Allows for events that are only placeable by admins
//Once placed by an admin, anyone can activate them
//If desired, event developers must add an if admin check inside individual events functions to prevent this

//To use, call registerAdminOnlyOutputEvent after calling registerOutputEvent
//registerAdminOnlyOutputEvent(%class,%name,%adminLevel);
//%class - the class of your event's target. FxDtsBrick, Player, GameConnection, etc
//%name - the name of your output event
//%adminLevel - admin level required to place the event. 1 for admin, 2 for superadmin, 3 for host

$AdminLevelList = "Normal Admin SuperAdmin Host";

if($AdminOutputEventCount $= "") //A bit of a hack...if the variable hasn't been touched yet,
	$AdminOutputEventCount = -1;	//Set it to -1 so the index starts at 0

package AdminEvents
{
	function serverCmdAddEvent(%client, %delay, %input, %target, %a, %b, %output, %par1, %par2, %par3, %par4)
	{
		//Define aLevel based on whether the client is host, super admin, admin, or none
		if(%client.bl_id == getNumKeyID())
			%aLevel = 3;
		else if(%client.isSuperAdmin)
			%aLevel = 2;
		else if(%client.isAdmin)
			%aLevel = 1;
		else
			%aLevel = 0;

		//Get more information about what event the client chose
		%class = getWord(getField($InputEvent_TargetListfxDTSBrick_[%input],%a),1);
		%name = $OutputEvent_Name[%class, %output];
		%reqLevel = getWord($AdminOutputEvent[%class,%name],0);
		
		//Does the client have sufficient priviledges to use this event?
		if(%reqLevel > %aLevel)
			messageClient(%client,'','You do not have a sufficient admin level to use the event %1::%2. It has been removed from your brick. Type \c2/restrictedEvents \c0for more info',%class,%name);
		else
			Parent::serverCmdAddEvent(%client, %delay, %input, %target, %a, %b, %output, %par1, %par2, %par3, %par4);
	}
	
	//Fix a duplicator exploit that allows non-admins to duplicate bricks with admin only events on them
	function fxDtsBrick::onPlant(%this)
	{
		%r = Parent::onPlant(%this);
		%this.schedule(50,checkForAdminEvents);
		return %r;
	}
	
	//Warn the client when they join that some events may not be usable by them
	function GameConnection::AutoAdminCheck(%this)
	{
		//messageClient(%this,'','Some events on this server may be restricted to you depending on your admin level. Type \c2/restrictedEvents \c0for more info');
		return Parent::AutoAdminCheck(%this);
	}
};
ActivatePackage(AdminEvents);

function registerAdminOnlyOutputEvent(%class,%name,%adminLevel)
{
	//Add the event to the list of admin-only events
	%id = outputEvent_GetOutputEventIdx(%class,%name);
	if(%id == -1) //This could be caused by typos in the event class or event name, or calling before registering the event
		error("registerAdminOnlyOutputEvent - Output event " @ %class @ "::" @ %name @ " not found!");
	else if(%adminLevel < 1 || %adminLevel > 3)
	{
		error("registerAdminOnlyOutputEvent - Invalid adminLevel: " @ %adminLevel);
	}
	else
	{
		if($AdminOutputEvent[%class,%name] $= "") //Don't increase the count of admin only events if we've already defined this one
		{
			$AdminOutputEventList[$AdminOutputEventCount++] = %class @ "::" @ %name SPC %adminLevel; //Define one array that we can easily loop through...
			$AdminOutputEvent[%class,%name] = %adminLevel SPC $AdminOutputEventCount;	//and another that we can find what we need quickly without looping through everything
		}
		else
			error("registerAdminOnlyOutputEvent - Output event " @ %class @ "::" @ %name @ " already registered!");
	}
	
}

//Function used to check for admin-only events after duplicating
function fxDtsBrick::checkForAdminEvents(%this)
{
	%client = %this.getGroup().client;
	if(%client.bl_id == getNumKeyID())
		%aLevel = 3;
	else if(%client.isSuperAdmin)
		%aLevel = 2;
	else if(%client.isAdmin)
		%aLevel = 1;
	else
		%aLevel = 0;
	for(%i=0;%i<%this.numEvents;%i++)
	{
		
		%event = InputEvent_GetInputEventIdx(%this.eventInput0);
		%class = getWord(getField($InputEvent_TargetListFxDtsBrick_[%event],%this.eventTargetIdx[%i]),1);
		%name = %this.eventOutput[%i];
		%reqLevel = getWord($AdminOutputEvent[%class,%name],0);
		if(%reqLevel > %aLevel) 
			%this.eventEnabled[%i] = 0; //Just disabling the event is plenty. It will be deleted when the client resends it after wrenching it
	}
}

function serverCmdRestrictedEvents(%client)
{
	if(%client.bl_id == getNumKeyID())
		%aLevel = 3;
	else if(%client.isSuperAdmin)
		%aLevel = 2;
	else if(%client.isAdmin)
		%aLevel = 1;
	else
		%aLevel = 0;
		
	for(%i=%aLevel+1;%i<=3;%i++)
	{
		%message = "d \c2" @ getWord($AdminLevelList,%i) @ "-Only"; //The d at the beginning is another small hack due to the way the whole message is sent in pieces
		%eventsAdded = 0;
		for(%j=0;%j<=$AdminOutputEventCount;%j++)
		{
			%event = $AdminOutputEventList[%j];
			%name = getWord(%event,0);
			%reqLevel = getWord(%event,1);
			if(%reqLevel == %i && %aLevel < %reqLevel)
			{
				%eventsAdded = 1;
				%anyEventsAdded = 1;
				%message = %message SPC "\c3" @ %name;
			}
		}
		
		//Break long messages apart into several pieces, and send them, so we don't exceed max length
		if(%eventsAdded)
		{
			%wc = getWordCount(%message);
			for(%j=0;%j<%wc/7;%j++)
			{
				%piece = getWords(%message,%j*7+1,%j*7+7);
				messageClient(%client,'',%piece);
			}
		}
	}
	if(!%anyEventsAdded)
		messageClient(%client,'','\c3No events are restricted for your admin level');
}